from datetime import datetime

from nbg_currency_api.api import CurrencyFetcherAPI
from nbg_currency_api.types import (
    ClientModeEnum,
    CurrencyDataResponse,
    CurrencyRateItem,
    CurrencyEnum,
)


class NBGCurrencyService:
    def __init__(
        self,
        date: datetime | None = None,
        currency: CurrencyEnum | None = None,
        mode: ClientModeEnum = ClientModeEnum.SYNC,
    ):
        self.date = date or datetime.now()
        self.currency = currency
        self.mode = mode
        self.api = CurrencyFetcherAPI(self.date, self.currency, self.mode)

    @staticmethod
    def _normalize_data(data: dict) -> CurrencyDataResponse:
        first_item = data[0]
        return CurrencyDataResponse(
            date=datetime.fromisoformat(first_item["date"]),
            currencies=[
                CurrencyRateItem(
                    code=currency["code"],
                    quantity=currency["quantity"],
                    rateFormated=currency["rateFormated"],
                    diffFormated=currency["diffFormated"],
                    rate=currency["rate"],
                    name=currency["name"],
                    diff=currency["diff"],
                    date=datetime.fromisoformat(currency["date"]),
                    validFromDate=datetime.fromisoformat(currency["validFromDate"]),
                )
                for currency in first_item["currencies"]
            ],
        )

    def fetch(self) -> CurrencyDataResponse:
        raw_data = self.api.fetch()
        return self._normalize_data(raw_data)

    async def afetch(self) -> CurrencyDataResponse:
        raw_data = await self.api.afetch()
        return self._normalize_data(raw_data)



import requests

try:
    import aiohttp
except ImportError:
    aiohttp = None

from datetime import datetime

from nbg_currency_api.exceptions import CurrencyAPIException
from nbg_currency_api.types import (
    CurrencyEnum,
    ClientModeEnum,
)
from nbg_currency_api.constants import BASE_URL


class CurrencyFetcherAPI:
    def __init__(
        self,
        date: datetime | None = None,
        currency: CurrencyEnum | None = None,
        mode: ClientModeEnum = ClientModeEnum.SYNC,
    ):
        self.date = date or datetime.now()
        self.currency = currency
        self.mode = mode
        if self.mode == ClientModeEnum.ASYNC and aiohttp is None:
            raise ImportError(
                "aiohttp is not installed. Install with 'pip install currency-rates[async]' to enable async mode."
            )

    def _build_url(self) -> str:
        formatted_date = self.date.strftime("%Y-%m-%d")
        if self.currency:
            return f"{BASE_URL}/en/json/?currencies={self.currency.value}&date={formatted_date}"
        return f"{BASE_URL}/en/json/?date={formatted_date}"

    def fetch(self) -> dict:
        url = self._build_url()
        response = requests.get(url)
        if not response.ok:
            raise CurrencyAPIException(
                f"Error fetching data: {response.status_code} {response.text}"
            )
        return response.json()

    async def afetch(self) -> dict:
        url = self._build_url()
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                if response.status != 200:
                    raise CurrencyAPIException(
                        f"Error fetching data: {response.status} {await response.text()}"
                    )
                return await response.json()



from nbg_currency_api.services import NBGCurrencyService
from nbg_currency_api.types import (
    CurrencyEnum,
    CurrencyRateItem,
    CurrencyDataResponse,
)


__all__ = (
    "NBGCurrencyService",
    "CurrencyDataResponse",
    "CurrencyRateItem",
    "CurrencyEnum",
)



from setuptools import setup, find_packages

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name="nbg-currencies-api",
    version="0.1.3",
    description="API wrapper for fetching and normalizing NBG currency rates",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Illia Stebelskyi",
    author_email="illia.stebelski@gmail.com",
    license="MIT",
    packages=find_packages(),
    install_requires=["requests"],
    extras_require={
        "async": ["aiohttp"],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    project_urls={
        "Source": "https://github.com/zelnkup/nbg-currencies-api",
        "Tracker": "https://github.com/zelnkup/nbg-currencies-api/issues",
    },
)



from dataclasses import dataclass
from typing import List
from datetime import datetime
from enum import Enum


@dataclass
class CurrencyRateItem:
    code: str
    quantity: int
    rateFormated: str
    diffFormated: str
    rate: float
    name: str
    diff: float
    date: datetime
    validFromDate: datetime


@dataclass
class CurrencyDataResponse:
    date: datetime
    currencies: List[CurrencyRateItem]


class ClientModeEnum(Enum):
    SYNC = "sync"
    ASYNC = "async"


class CurrencyEnum(Enum):
    AED = "AED"
    AMD = "AMD"
    AUD = "AUD"
    AZN = "AZN"
    BGN = "BGN"
    BRL = "BRL"
    BYN = "BYN"
    CAD = "CAD"
    CHF = "CHF"
    CNY = "CNY"
    CZK = "CZK"
    DKK = "DKK"
    EGP = "EGP"
    EUR = "EUR"
    GBP = "GBP"
    HKD = "HKD"
    HUF = "HUF"
    ILS = "ILS"
    INR = "INR"
    IRR = "IRR"
    ISK = "ISK"
    JPY = "JPY"
    KGS = "KGS"
    KRW = "KRW"
    KWD = "KWD"
    KZT = "KZT"
    MDL = "MDL"
    NOK = "NOK"
    NZD = "NZD"
    PLN = "PLN"
    QAR = "QAR"
    RON = "RON"
    RSD = "RSD"
    RUB = "RUB"
    SEK = "SEK"
    SGD = "SGD"
    TJS = "TJS"
    TMT = "TMT"
    TRY = "TRY"
    UAH = "UAH"
    USD = "USD"
    UZS = "UZS"
    ZAR = "ZAR"
